from re import match
import dateutil.parser
from mwparserfromhell.nodes import Template
from pytz import timezone
from .wiki_time import WikiTime


def time_from_str(timestamp: str, tz: timezone = None):
    timestamp_parsed = dateutil.parser.parse(timestamp)
    return WikiTime(timestamp_parsed, tz=tz)


def time_from_template(template: Template):
    """
    Pulls date-time information encoded by a template and returns a WikiTime object.
    If date-time information is missing or incomplete, None is returned instead.
    
    :param template: A mwparserfromhell Template object with date, time, and timezone parameters
    :return: a WikiTime object representing the date-time information encoded by this template
    """
    tz_lookup = {
        'PST': WikiTime.pst,
        'CET': WikiTime.cet,
        'KST': WikiTime.kst
    }
    if not template.has('date') or not template.has('time'):
        return None
    date = template.get("date").value.strip()
    time = template.get("time").value.strip()
    if date == '' or time == '':
        return None

    # Fix case of a time being written as 100 or 1100 without a :
    if match(r'\d\d\d\d', time):
        time = '{}:{}'.format(time[:2], time[3:])
    elif match(r'\d\d\d', time):
        time = '{}:{}'.format(time[:1], time[2:])

    tz_local_str = template.get('timezone').value.strip()
    tz_local = tz_lookup[tz_local_str]
    date_and_time = date + " " + time
    return time_from_str(date_and_time, tz=tz_local)



import datetime
import pytz


class WikiError(object):
    error_type = None
    title = None
    error = None
    
    def __init__(self):
        self.timestamp = datetime.datetime.now(tz=pytz.timezone('America/Los_Angeles'))
        self.date = self.timestamp.strftime('%Y-%m-%d')
    
    def format_for_print(self):
        return '{} - {}: {} - {}'.format(
            self.date,
            self.error_type,
            '[[{}]]'.format(self.title) if self.title else '(No title recorded)',
            self.error
        )



from .wiki_error import WikiError


class WikiContentError(WikiError):
    """Exceptions that we note based on the content of the wiki
    """
    
    def __init__(self, title: str = None, error: str = None):
        super().__init__()
        self.title = title
        self.error = error if error else 'No details provided'
        self.error_type = 'Wiki Content Error'



from typing import Union, Optional

from mwcleric.template_modifier import TemplateModifierBase as MwclericTemplateModifier
from mwparserfromhell.nodes import Template

from mwrogue.esports_client import EsportsClient


class TemplateModifierBase(MwclericTemplateModifier):
    def __init__(self, site: EsportsClient, template, page_list=None, title_list=None, limit=-1, summary=None,
                 quiet=False, lag=0, tags=None, skip_pages=None,
                 recursive=True,
                 startat_page=None,
                 namespace: Optional[Union[int, str]] = None,
                 **data):
        super().__init__(site, template, page_list=page_list, title_list=title_list, limit=limit, summary=summary,
                         quiet=quiet, lag=lag, tags=tags, skip_pages=skip_pages,
                         recursive=recursive,
                         startat_page=startat_page,
                         namespace=namespace,
                         **data)

        # redo this assignment just for the type hint because it doesn't seem to get it otherwise
        self.site = site

    def backup(self, key):
        if self.current_page.name.startswith('Backup:'):
            return
        self.site.backup_template(template=self.current_template, page=self.current_page, key=key)

    def restore(self, key):
        self.current_template: Template
        if self.current_page.name.startswith('Backup:'):
            return
        to_restore = self.site.get_restored_template(self.current_template, self.current_page, key)
        if to_restore is None:
            if not self.quiet:
                if isinstance(key, str):
                    key = [key]
                print('Could not find restore data for template on page "{}" with key: {}'.format(
                    self.current_page.name,
                    ', '.join([str(self.current_template.get(_, _)) for _ in key])))
            return
        to_restore.remove('backup_key')

        for param in self.current_template.params:
            self.current_template.remove(param.name.strip())

        for param in to_restore.params:
            name = param.name.strip()
            self.current_template.add(param.name, to_restore.get(name).value, preserve_spacing=False)



from pytz import timezone, utc
from datetime import datetime


class WikiTime(object):
    """
    Leaguepedia and the other esports wikis use an EXTREMELY simplified time zone model.
    In this model, there are only three time zones: PST, CET, and KST.
    Additionally, the wiki itself does not know anything about daylight savings;
    instead, it relies on the user supplying DST metadata along with every timestamp.
    DST metadata can be supplied with three options: Yes, Spring [or Fall], or No.
    Yes means that both CET and PST have DST; No means that neither does, and
    Spring/Fall mean that PST does but CET does not. (KST, sanely, never has DST.)
    For simplicity, even in the fall, we return `Spring`.
    
    This class reduces timestamps to this simple model and provides information
    in the format expected by the esports wikis.
    """
    pst = timezone('America/Los_Angeles')
    cet = timezone('Europe/Berlin')
    kst = timezone('Asia/Seoul')
    
    def __init__(self, timestamp: datetime, tz: timezone = utc):
        """
        Creates a wiki date-time object
        :param timestamp: a datetime object
        :param tz: optional, a timezone. if not provided, utc will be assumed.
        """
        if timestamp.tzinfo is None:
            timestamp = tz.localize(timestamp)
        self.pst_object = timestamp.astimezone(self.pst)
        self.cet_object = timestamp.astimezone(self.cet)
        self.kst_object = timestamp.astimezone(self.kst)
        self.pst_date = self.pst_object.strftime('%Y-%m-%d')
        self.cet_date = self.cet_object.strftime('%Y-%m-%d')
        self.kst_date = self.kst_object.strftime('%Y-%m-%d')
        self.pst_time = self.pst_object.strftime('%H:%M')
        self.cet_time = self.cet_object.strftime('%H:%M')
        self.kst_time = self.kst_object.strftime('%H:%M')
        self.dst = self._determine_dst()
    
    def _determine_dst(self):
        is_dst_pst = self.pst_object.dst()
        is_dst_cet = self.cet_object.dst()
        if is_dst_pst and is_dst_cet:
            return 'yes'
        elif is_dst_pst:
            return 'spring'
        else:
            return 'no'



from .wiki_error import WikiError


class WikiScriptError(WikiError):
    """Exceptions raised by scripts
    """
    
    def __init__(self, title: str = None, error: Exception = None):
        super().__init__()
        self.title = title
        self.error = error
        self.error_type = type(self.error)



import copy
import json
import re

import mwparserfromhell
from mwclient.page import Page
from typing import List, Union, Optional

from mwparserfromhell.nodes.extras import Parameter

from .auth_credentials import AuthCredentials
from mwcleric.clients.cargo_client import CargoClient

from .error_reporting.wiki_content_error import WikiContentError
from .error_reporting.wiki_script_error import WikiScriptError
from .errors import CantFindMatchHistory
from .lookup_cache import EsportsLookupCache
from mwcleric.fandom_client import FandomClient
from mwcleric.clients.site import Site
from mwparserfromhell.nodes.template import Template

ALL_ESPORTS_WIKIS = ['lol', 'halo', 'smite', 'vg', 'rl', 'pubg', 'fortnite',
                     'apexlegends', 'fifa', 'gears', 'nba2k', 'paladins', 'siege',
                     'splatoon2', 'legendsofruneterra',
                     'default-loadout', 'commons', 'teamfighttactics', 'valorant']


class EsportsClient(FandomClient):
    """
    Functions for connecting to and editing specifically to Gamepedia esports wikis.

    If not using an esports wiki, please use GamepediaSite instead.
    """
    ALL_ESPORTS_WIKIS = ALL_ESPORTS_WIKIS
    cargo_client: CargoClient = None
    client: Site = None
    wiki: str = None

    def __init__(self, wiki: str, client: Site = None,
                 credentials: AuthCredentials = None,
                 cache: EsportsLookupCache = None,
                 lang: str = None,
                 **kwargs):
        """
        Create a site object.
        :param wiki: Name of a wiki
        :param client: WikiClient object. If this is provided, SessionManager will not be used.
        :param credentials: Optional. Provide if you want a logged-in session.
        :param stg: if it's a staging wiki or not
        """
        self.wiki = self.get_wiki(wiki)

        super().__init__(wiki, credentials=credentials, lang=lang, client=client, **kwargs)
        if cache:
            self.cache = cache
        else:
            self.cache = EsportsLookupCache(self.client, cargo_client=self.cargo_client)
        self.errors = []

    @staticmethod
    def get_wiki(wiki):
        if wiki in ['lol', 'teamfighttactics'] or wiki not in ALL_ESPORTS_WIKIS:
            return wiki
        return wiki + '-esports'

    def setup_tables(self, tables):
        if isinstance(tables, str):
            tables = [tables]
        summary = "Setting up Cargo declaration"
        for table in tables:
            tl_page = self.client.pages['Template:{}/CargoDec'.format(table)]
            doc_page = self.client.pages['Template:{}/CargoDec/doc'.format(table)]
            self.save(
                tl_page,
                '{{Declare|doc={{{1|}}}}}<noinclude>{{documentation}}</noinclude>',
                summary=summary
            )
            self.save(doc_page, '{{Cargodoc}}', summary=summary)
            tl_page.touch()
        self.create_tables(tables)
        for table in tables:
            self.client.pages['Template:{}/CargoDec'.format(table)].touch()

    def create_tables(self, tables):
        self.recreate_tables(tables, replacement=False)

    def recreate_tables(self, tables, replacement=True):
        if isinstance(tables, str):
            tables = [tables]
        templates = ['{}/CargoDec'.format(_) for _ in tables]
        self.cargo_client.recreate(templates, replacement=replacement)

    def get_one_data_page(self, event, i):
        """
        Find one data page for an event
        :param event: Overview Page of an event
        :param i: the ith page to return
        :return: a Page object of a single data page
        """
        if i == 1:
            return self.client.pages['Data:' + event]
        return self.client.pages['Data:{}/{}'.format(event, str(i))]

    def data_pages(self, event):
        """
        Find all the data pages for an event.
        :param event: Overview Page of event
        :return: generator of data pages
        """
        event = self.cache.get_target(event)
        i = 1
        data_page = self.get_one_data_page(event, i)
        while data_page.exists:
            yield data_page
            i += 1
            data_page = self.get_one_data_page(event, i)

    def query_riot_mh(self, riot_mh):
        match = re.search(r'match-details/(.+?)(&tab=.*)?$', riot_mh)
        if match[1] is None:
            raise CantFindMatchHistory
        to_search = '%{}%'.format(match[1])
        result = self.cargo_client.query(
            tables="MatchScheduleGame=MSG, Tournaments=T, MatchSchedule=MS",
            join_on="MSG.OverviewPage=T.OverviewPage, MSG.MatchId=MS.MatchId",
            fields="T.StandardName=Event, MSG.Blue=Blue, MSG.Red=Red, MS.Patch=Patch",
            where="MSG.MatchHistory LIKE\"{}\"".format(to_search)
        )
        if len(result) == 0:
            raise CantFindMatchHistory
        return result[0]

    def query_bayes_id(self, idx):
        result = self.cargo_client.query(
            tables="MatchScheduleGame=MSG, Tournaments=T, MatchSchedule=MS",
            join_on="MSG.OverviewPage=T.OverviewPage, MSG.MatchId=MS.MatchId",
            fields="MS.Patch=Patch, T.StandardName=Event",
            where="MSG.RiotPlatformGameId=\"{}\"".format(idx)
        )
        if len(result) == 0:
            raise CantFindMatchHistory
        return result[0]

    def query_qq_mh(self, qq_id):
        result = self.cargo_client.query(
            tables="MatchSchedule=MS, Tournaments=T",
            join_on="MS.OverviewPage=T.OverviewPage",
            fields="MS.Patch=Patch, T.StandardName=Event",
            where="MS.QQ=\"{}\"".format(qq_id)
        )
        if len(result) == 0:
            raise CantFindMatchHistory
        return result[0]

    def query_wp_mh(self, wp_id):
        result = self.cargo_client.query(
            tables="MatchSchedule=MS, Tournaments=T",
            join_on="MS.OverviewPage=T.OverviewPage",
            fields="MS.Patch=Patch, T.StandardName=Event",
            where="MS.WanplusId=\"{}\"".format(wp_id)
        )
        if len(result) == 0:
            raise CantFindMatchHistory
        return result[0]

    def get_v4_data_and_timeline(self, rpgid: str):
        """
        Queries Leaguepedia to return two jsons: The data & timeline from a single game.

        This function is limited in scope: It will not allow you to query multiple games in a single query;
        however, the MediaWiki API does support this. It also will not allow you to drop one of the jsons
        for a smaller response package if you don't require all of the data. You also must know the ID in advance.
        You can find IDs by querying the MatchScheduleGame Cargo table and looking up the RiotPlatformGameId field.

        Raises a KeyError in the case that data is not found.
        If Timeline is not found, `None` will be returned for that json (this happens for chronobreaks).

        This function is unavailable on wikis other than Leaguepedia.
        :param rpgid: A single riot_platform_game_id
        :return: Two jsons, the data & timeline for the game
        """
        titles = 'V4 data:{}|V4 data:{}/Timeline'.format(rpgid, rpgid)
        result = self.client.post(
            'query', prop='revisions', titles=titles, rvprop='content',
            rvslots='main'
        )
        data = None
        timeline = None
        for _, page_data in result['query']['pages'].items():
            # This is lazy but there's 2 pages total so it's safe tbh
            if 'Timeline' in page_data['title']:
                timeline = json.loads(page_data['revisions'][0]['slots']['main']['*'])
            else:
                data = json.loads(page_data['revisions'][0]['slots']['main']['*'])

        if data is None:
            raise KeyError
        return data, timeline

    def backup_template(self, template: Template, page: Union[str, Page],
                        key: Union[str, List[str]]):
        """
        Backs up a template in the `Backup` namespace. The template can later be restored with `get_restored_template`.
        :param template: Template object
        :param page: Page or title where the template is located on
        :param key: Identifying set of params that we can use to locate the template when we restore it
        :return: null
        """
        if isinstance(page, str):
            page = self.client.pages[page]
        if isinstance(key, str):
            key = [key]
        key_template = Template('BackupKey')
        for key_param in key:
            key_template.add(key_param, template.get(key_param, Parameter('', '')).value)

        # this method will be used in TemplateModifier so it is essential that
        # we do not modify the original
        copy_template = copy.deepcopy(template)
        copy_template.add('backup_key', str(key_template))
        self.client.pages['Backup:' + page.name].append('\n' + str(copy_template), contentmodel='text')

    def get_restored_template(self, template: Template, page: Union[str, Page],
                              key: Union[str, List[str]]) -> Optional[Template]:
        """
        Looks for the backed-up version of the specified template on the backup page & returns it

        The template should have been backed up using the backup_template method earlier.

        :param template: Template object that we want to restore from backup page
        :param page: Page or title where the template is located on
        :param key: Identifying set of params to use to restore the template from
        :return: Template object, if found, else None
        """
        if isinstance(page, str):
            page = self.client.pages[page]
        if isinstance(key, str):
            key = [key]
        backup_text = self.client.pages['Backup:' + page.name].text()
        for backup_template in mwparserfromhell.parse(backup_text).filter_templates():
            if not backup_template.name.matches(template.name):
                continue

            # kinda need to do a hack to get this as a template
            backup_key_str = str(backup_template.get('backup_key').value)
            backup_key_wikitext = mwparserfromhell.parse(backup_key_str)
            backup_key = None
            for tl in backup_key_wikitext.filter_templates():
                if tl.name.matches('BackupKey'):
                    backup_key = tl
                    break
            # now backup_key is a template value of BackupKey

            is_match = True
            i = 0
            for param in backup_key.params:
                name = param.name.strip()
                if name in key:
                    if param.value.strip() == template.get(name, Parameter('', '')).value.strip():
                        i += 1
                else:
                    is_match = False
            if i == len(key) and is_match:
                return backup_template
        return None

    def log_error_script(self, title: str = None, error: Exception = None):
        self.errors.append(WikiScriptError(title, error))

    def log_error_content(self, title: str = None, text: str = None):
        self.errors.append(WikiContentError(title, error=text))

    def report_all_errors(self, error_title):
        if not self.errors:
            return
        error_page = self.client.pages['Log:' + error_title]
        errors = [_.format_for_print() for _ in self.errors]
        error_text = '<br>\n'.join(errors)
        error_page.append('\n' + error_text)

        # reset the list so we can reuse later if needed
        self.errors = []



class EsportsCacheKeyError(KeyError):
    def __init__(self, file, value, length, value_table):
        self.file = file
        self.value = value
        self.length = length
        self.value_table = value_table
        self.allowed_keys = value_table.keys()

    def __str__(self):
        return "Invalid length of {} requested for {} in {}. Allowed: {}".format(
            self.length,
            self.value,
            self.file,
            ', '.join(self.allowed_keys)
        )


class CantFindMatchHistory(KeyError):
    def __str__(self):
        return "Cannot find any valid tournament for provided match history. It may be missing from the MatchSchedule data, or there may be an issue with the parser."


class InvalidEventError(KeyError):
    def __str__(self):
        return "Invalid page name provided for event"



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mwrogue",
    version="0.0.11",
    author="RheingoldRiver",
    author_email="river.esports@gmail.com",
    description="Client for accessing Fandom/Gamepedia Esports Wikis",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/RheingoldRiver/mwrogue",
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=['mwparserfromhell', 'pytz', 'mwclient>=0.10.1', 'python-dateutil', 'Unidecode', 'mwcleric>=0.6.2']
)



import json
import re

from unidecode import unidecode

from mwcleric.clients.cargo_client import CargoClient
from .errors import EsportsCacheKeyError, InvalidEventError
from mwcleric.clients.site import Site


class EsportsLookupCache(object):
    def __init__(self, site: Site, cargo_client: CargoClient = None):
        self.site = site
        self.cargo_client = cargo_client
        self.cache = {}
        self.redirect_cache = {}
        self.event_tricode_cache = {}
        self.event_playername_cache = {}

    def clear(self):
        self.cache = {}
        self.redirect_cache = {}
        self.event_tricode_cache = {}
        self.event_playername_cache = {}

    def _get_json_lookup(self, filename):
        """
        Returns a json representation of the requested file, queriying the site to retrieve it if needed
        :param filename: The name of the file to return, e.g. "Champion" or "Role"
        :return: A json object representing the lookup file
        """
        if filename in self.cache:
            return self.cache[filename]
        # this compartmentalization is in place for Module:Teamnames, whose halfway point is somewhere in the middle
        # of the letter T because of `Team` so a-s not a-m
        dict1 = self._get_one_encoded_json(filename, 'include_match=^[a-s].*')
        dict2 = self._get_one_encoded_json(filename, 'exclude_match=^[a-s].*')
        self.cache[filename] = {**dict1, **dict2}
        return self.cache[filename]

    def _get_one_encoded_json(self, filename, mask):
        result = self.site.api(
            'expandtemplates',
            prop='wikitext',
            text='{{{{JsonEncode|{}|{}}}}}'.format(filename, mask)
        )
        return json.loads(result['expandtemplates']['wikitext'])

    def get(self, filename, key, length, allow_fallback=False):
        """
        Returrns the length of the lookup of a key requested from the filename requested. Assumes the file has
        the same structure as the -names modules on Leaguepedia.
        :param filename: "Champion", "Role", etc. - the name of the file
        :param key: The lookup key, e.g. "Morde"
        :param length: The length of value to return, e.g. "long" or "link"
        :param allow_fallback: Whether or not to fallback to returning the key if it's missing in the lookup
        :return: Correct lookup value provided, or None if it's not found
        :return: Correct lookup value provided, or None if it's not found
        """
        file = self._get_json_lookup(filename)
        if key is None:
            return None
        key = key.lower()
        if key not in file:
            if allow_fallback:
                return key
            return None
        value_table = file[key]
        if isinstance(value_table, str):
            key = value_table
            value_table = file[value_table]
        if length not in value_table:
            raise EsportsCacheKeyError(filename, key, length, value_table)
        return value_table[length]

    def get_target(self, title):
        """
        Caches & returns the target of a title of a wiki page, caching the result and returning
        the cached result if possible
        :param title: Title of a page on the wiki
        :return: Redirect target of the title
        """
        title = title.replace('_', ' ')
        if title in self.redirect_cache:
            return self.redirect_cache[title]
        return self.site.pages[title].resolve_redirect().name

    def get_team_from_event_tricode(self, event, tricode):
        """
        Determines the full name of a team based on its tricode, assuming tricode matches the short name on the wiki
        and that tricodes are unique within the provided event
        :param event: Event within which to restrict the lookup
        :param tricode: Official tricode of the team, must match wiki teamshort
        :return: Wiki teamlinkname
        """
        if event is None:
            raise InvalidEventError
        event = self.get_target(event)
        tricode = tricode.lower()
        result = self._get_team_from_event_tricode_raw(event, tricode)
        if result is not None:
            return result
        self._populate_event_tricodes(event)
        return self._get_team_from_event_tricode_raw(event, tricode)

    def _get_team_from_event_tricode_raw(self, event, tricode):
        if event in self.event_tricode_cache:
            if tricode in self.event_tricode_cache[event]:
                return self.event_tricode_cache[event][tricode]
        return None

    def _populate_event_tricodes(self, event):
        result = self.cargo_client.query(
            tables="TournamentRosters=Ros,TeamRedirects=TRed,Teams",
            join_on="Ros.Team=TRed.AllName,TRed._pageName=Teams.OverviewPage",
            where='Ros.OverviewPage="{}"'.format(event),
            fields="Ros.Team=Team, COALESCE(Ros.Short,Teams.Short)=Short"
        )
        d = {}
        for item in result:
            team = item['Team']
            link = self.unescape(self.get('Team', team, 'link', allow_fallback=True))
            short = self.unescape(item['Short'])
            if short == '':
                short = self.get('Team', team, 'short')
            if short is not None and short != '':
                d[short.lower()] = link
        self.event_tricode_cache[event] = d

    @staticmethod
    def unescape(string):
        return string.replace('&amp;', '&')

    def get_disambiguated_player_from_event(self, event, team, player):
        """
        Returns the disambiguated ID of the player based on the team they were playing on in the event.

        For performance, the first time a team & event are queried, a single network call will be made
        to retrieve all possible player names for that event.

        These will be stored in a three-layer dictionary keyed first by event, then by team,
        then finally by player ID (not disambiguated), and ultimately yielding disambiguated player ID
        Because it's possible for a player to rename mid-event, we will just include every lifetime ID
        the player has ever had, so that future requests in the same session can use other IDs.
        The current request will use the ID requested by the current function call.

        This method has a failure chance if two players on the same team historically shared an ID
        We could attempt to mitigate this failure chance by checking position
        TODO: Add support for low-priority disambiguation table, this will also mitigate this possibility

        teams are themselves listed within tournaments
        :param event: will be resolved as a redirect if needed
        :param team: can be a tricode if needed
        :param player: the current player ID to return the disambiguated name of
        :return: the disambiguated form of the player param
        """
        if event is None:
            raise InvalidEventError
        event = self.get_target(event)

        # we'll keep all player keys lowercase
        player_lookup = unidecode(player).lower()
        team = self.get('Team', team, 'link', allow_fallback=True)
        disambiguation = self._get_player_from_event_and_team_raw(event, team, player_lookup)
        if disambiguation is not None:
            return player + disambiguation
        self._populate_event_team_players(event)
        disambiguation = self._get_player_from_event_and_team_raw(event, team, player_lookup)
        if disambiguation is not None:
            return player + disambiguation
        return None

    def _get_player_from_event_and_team_raw(self, event, team, player_lookup):
        if event in self.event_playername_cache:
            if team in self.event_playername_cache[event]:
                if player_lookup in self.event_playername_cache[event][team]:
                    return self.event_playername_cache[event][team][player_lookup]
        return None

    def _populate_event_team_players(self, event):
        result = self.cargo_client.query(
            tables="TournamentPlayers=TP,PlayerRedirects=PR1,PlayerRedirects=PR2,LowPriorityRedirects=LPR",
            join_on="TP.Player=PR1.AllName,PR1.OverviewPage=PR2.OverviewPage,PR2.AllName=LPR._pageName",
            where="TP.OverviewPage=\"{}\" AND LPR.IsLowPriority IS NULL".format(event),
            fields="TP.Team=Team,PR2.AllName=DisambiguatedName,PR2.ID=ID,TP.Player=TournamentName,PR2.OverviewPage=CurrentName"
        )
        d = {}
        for item in result:
            if item['Team'] not in d:
                d[item['Team']] = {}
            team_entry = d[item['Team']]
            if unidecode(item['ID']) == unidecode(item['DisambiguatedName']):
                item['DisambiguatedName'] = item['ID']
            disambiguation = re.sub(r'^' + re.escape(item['ID']), '', item['DisambiguatedName'])
            key = unidecode(item['ID']).lower()
            if key not in team_entry or disambiguation != '':
                team_entry[key] = disambiguation
        self.event_playername_cache[event] = d
